@@ -1220,6 +1220,11 @@ svn_path_canonicalize(const char *path, apr_pool_t *pool)
   apr_size_t canon_segments = 0;
   svn_boolean_t uri;
 
+  /* "" is already canonical, so just return it; note that later code
+     depends on path not being zero-length.  */
+  if (! *path)
+    return path;
+
   dst = canon = apr_pcalloc(pool, strlen(path) + 1);
 
   /* Copy over the URI scheme if present. */
@@ -1281,9 +1286,17 @@ svn_path_canonicalize(const char *path, apr_pool_t *pool)
         src++;
     }
 
-  /* Remove the trailing slash. */
-  if ((canon_segments > 0 || uri) && *(dst - 1) == '/')
-    dst--;
+  /* Remove the trailing slash if necessary. */
+  if (*(dst - 1) == '/')
+    {
+      /* If we had any path components, we always remove the trailing slash. */
+      if (canon_segments > 0)
+        dst --;
+      /* Otherwise, make sure to strip the third slash from URIs which
+       * have an empty hostname part, such as http:/// or file:/// */
+      else if (uri && strcmp(skip_uri_scheme(canon), "/") == 0)
+        dst--;
+    }
 
   *dst = '\0';
 
